ACCEPTED_ERROR = "The {field} field must be accepted."
AFTER_ERROR = "The {field} field must be a date/time after {other}."
ALPHA_NUM_ERROR = "The {field} field can only contain letters and numbers."
ALPHA_NUM_SPACE_ERROR = "The {field} field can only contain letters, numbers and spaces."
BEFORE_ERROR = "The {field} field must be a date/time before {other}."
BOOLEAN_ERROR = "The {field} field must be a boolean."
DATE_TIME_ERROR = "The {field} field has to be a valid date/time matching the {format} format."
DECIMAL_ERROR = "The {field} field must be a decimal."
DICT_ERROR = "The {field} field must be a dict."
EMAIL_ERROR = "The {field} field has to be a valid email address."
ENDS_WITH_ERROR = "The {field} field must end with one of the following values: {values}."
FILLED_ERROR = "The {field} field must not be empty when it is present."
FLOAT_ERROR = "The {field} field must be a float."
IN_ERROR = "The {field} field must be one of the following values: {values}."
INTEGER_ERROR = "The {field} field must be an integer."
IP_ERROR = "The {field} field has to be a valid IP address."
JSON_ERROR = "The {field} field must be a valid JSON string."
LIST_ERROR = "The {field} field must be a list."
MAX_ERROR = "The {field} field has to have a maximum value of {max}."
MAX_ITEMS_ERROR = "The {field} field cannot contain more than {max} items."
MAX_STRING_ERROR = "The {field} field cannot be longer than {max} characters."
MIN_ERROR = "The {field} field has to have a minimum value of {min}."
MIN_ITEMS_ERROR = "The {field} field has to contain at least {min} items."
MIN_STRING_ERROR = "The {field} field has to be at least {min} characters."
NOT_WITH_ERROR = "The {field} field can't be present when the {other} field is present."
REQUIRED_ERROR = "The {field} field is required."
REQUIRED_IF_ERROR = "The {field} field is required if the {other} field equals {value}."
REQUIRED_UNLESS_ERROR = "The {field} field is required unless the {other} field equals {value}."
REQUIRED_WITH_ERROR = "The {field} field is required if any of the following fields are present: {other}."
REQUIRED_WITHOUT_ERROR = "The {field} field is required if any of the following fields are absent: {other}."
SIZE_ERROR = "The {field} field has to have a size of {size}."
STARTS_WITH_ERROR = "The {field} field must start with one of the following values: {values}."
STRING_ERROR = "The {field} field must be a string."
URL_ERROR = "The {field} field has to be a valid URL."
UUID4_ERROR = "The {field} field must be a valid UUID."
